import os
import shutil

from unittest import TestCase
from mock import Mock
import mock
import tempfile

from earthdiagnostics.data_convention import DataConvention
from earthdiagnostics.modelingrealm import ModelingRealms


class TestDataConvention(TestCase):

    def setUp(self):
        self.tmp_dir = tempfile.mkdtemp()
        os.mkdir(os.path.join(self.tmp_dir, 'expid'))

        self.config = Mock()
        self.config.experiment.experiment_name = 'expname'
        self.config.data_dir = self.tmp_dir
        self.config.experiment.expid = 'expid'
        self.config.cmor.activity = 'activity'
        self.config.experiment.institute = 'institute'
        self.config.experiment.model = 'model'
        self.config.cmor.append_startdate = False
        self.convention = DataConvention('name', self.config)

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def test_get_filename(self):
        with self.assertRaises(NotImplementedError):
            self.convention.get_file_name(None, None, None, None, None, None, None, None, None, None)

    def test_get_startdate_path(self):
        self.assertEqual(self.convention.get_startdate_path('19900101'),
                         os.path.join(self.tmp_dir, 'expid/cmorfiles/activity/institute/model/expname'))

    def test_experiment_name(self):
        self.assertEqual(self.convention.experiment_name('19900101'),
                         'expname')

    def test_experiment_name_append(self):
        self.config.cmor.append_startdate = True
        self.assertEqual(self.convention.experiment_name('19900101'),
                         'expnameS19900101')

    def test_get_cmor_folder_path(self):
        with self.assertRaises(NotImplementedError):
            self.convention.get_cmor_folder_path(None, None, None, None, None, None, None)

    def test_get_file_path_raise_incompatible_date_info(self):
        cmor_var = Mock()
        omon = Mock()
        omon.name = 'Omon'
        cmor_var.get_table.return_value = omon
        frequency = Mock()
        frequency.frequency = 'monthly'
        frequency.__str__ = Mock()
        frequency.__str__.return_value = 'frequency'

        self.assertRaises(ValueError, self.convention.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var',
                          cmor_var, None, frequency, year='1998')
        self.assertRaises(ValueError, self.convention.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var',
                          cmor_var, 1, frequency, year='1998')
        self.assertRaises(ValueError, self.convention.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var',
                          cmor_var, 1, frequency, date_str='1998')
        self.assertRaises(ValueError, self.convention.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var',
                          cmor_var, None, frequency, year='1998', date_str='1998')
        self.assertRaises(ValueError, self.convention.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var',
                          cmor_var, 1, frequency, year='1998', date_str='1998')
        self.assertRaises(ValueError, self.convention.get_file_path, '19900101', 1, ModelingRealms.ocean, 'var',
                          cmor_var, None, frequency)
